#ifndef __PS2MATRIX3x3_H__
#define __PS2MATRIX3x3_H__

#include "ps2vector3.h"

namespace KazMaths
{
	class Matrix3x3
	{
	public:
		Matrix3x3(void);
		Matrix3x3(const Matrix3x3 & rhs);
		Matrix3x3(float _elem[][3]);
		Matrix3x3(float _11, float _12, float _13,
				  float _21, float _22, float _23,
				  float _31, float _32, float _33);

		~Matrix3x3(void);

		inline float &operator()( unsigned int Row, unsigned int Column )
		{ return elem[Row][Column]; }
		inline float const &operator()( unsigned int Row, unsigned int Column ) const
		{ return elem[Row][Column]; }

		void Invert();

		Matrix3x3 GetInverse() const;

		float elem[3][3];

		static const Matrix3x3 IDENTITY;
		static const Matrix3x3 NULLMATRIX;
	};
}

KazMaths::Matrix3x3 operator * ( const KazMaths::Matrix3x3 & M1,
								   const KazMaths::Matrix3x3 & M2 );

KazMaths::Vector3 operator * ( const KazMaths::Matrix3x3 & M,
								 const KazMaths::Vector3 & V );

void Orthonormalise( KazMaths::Matrix3x3 & M );

// INLINE //

inline KazMaths::Matrix3x3 operator - ( const KazMaths::Matrix3x3 & M )
{
	return KazMaths::Matrix3x3(-M(0,0),-M(0,1),-M(0,2),
								 -M(1,0),-M(1,1),-M(1,2),
								 -M(2,0),-M(2,1),-M(2,2));
}
inline KazMaths::Matrix3x3 operator - ( const KazMaths::Matrix3x3 & M1,
										  const KazMaths::Matrix3x3 & M2 )
{
	return KazMaths::Matrix3x3(M1(0,0)-M2(0,0),M1(0,1)-M2(0,1),M1(0,2)-M2(0,2),
								 M1(1,0)-M2(1,0),M1(1,1)-M2(1,1),M1(1,2)-M2(1,2),
								 M1(2,0)-M2(2,0),M1(2,1)-M2(2,1),M1(2,2)-M2(2,2));
}

inline KazMaths::Matrix3x3 operator + ( const KazMaths::Matrix3x3 & M1,
										  const KazMaths::Matrix3x3 & M2 )
{
	return KazMaths::Matrix3x3(M1(0,0)+M2(0,0),M1(0,1)+M2(0,1),M1(0,2)+M2(0,2),
								 M1(1,0)+M2(1,0),M1(1,1)+M2(1,1),M1(1,2)+M2(1,2),
								 M1(2,0)+M2(2,0),M1(2,1)+M2(2,1),M1(2,2)+M2(2,2));
}

inline KazMaths::Matrix3x3 operator * ( const KazMaths::Matrix3x3 & M,
										  const float & s )
{
	return KazMaths::Matrix3x3(M(0,0) * s,M(0,1) * s,M(0,2) * s,
								 M(1,0) * s,M(1,1) * s,M(1,2) * s,
								 M(2,0) * s,M(2,1) * s,M(2,2) * s);
}

inline KazMaths::Matrix3x3 operator * ( const float & s,
										  const KazMaths::Matrix3x3 & M )
{
	return KazMaths::Matrix3x3(M(0,0) * s,M(0,1) * s,M(0,2) * s,
								 M(1,0) * s,M(1,1) * s,M(1,2) * s,
								 M(2,0) * s,M(2,1) * s,M(2,2) * s);
}

inline KazMaths::Matrix3x3 Transpose( KazMaths::Matrix3x3 const & M )
{
	KazMaths::Matrix3x3 ret;

	for(int j = 0; j < 3; j++)
	{
		for(int i = 0; i < 3; i++)
		{
			ret(i,j) = M(j,i);
		}
	}

	return ret;
}

#endif
